@@ -17,7 +17,7 @@
  * Copyright: 2005
  *     The copyright to this program is held by it's authors.
  *
- * ID: $Id: ClassUtil.java 1095 2006-07-23 19:15:47Z dmsmith $
+ * ID: $Id: ClassUtil.java 1167 2006-10-19 21:47:05Z dmsmith $
  */
 package org.crosswire.common.util;
 
@@ -65,13 +65,13 @@ public final class ClassUtil
             {
                 try
                 {
-                    String file_name = classname.replace(',', '/') + EXTENSION_CLASS;
+                    String fileName = classname.replace(',', '/') + EXTENSION_CLASS;
                     ZipFile zip = new ZipFile(paths[i]);
-                    ZipEntry entry = zip.getEntry(file_name);
+                    ZipEntry entry = zip.getEntry(fileName);
 
                     if (entry != null && !entry.isDirectory())
                     {
-                        if (full != null && !full.equals(file_name))
+                        if (full != null && !full.equals(fileName))
                         {
                             log.warn("Warning duplicate " + classname + " found: " + full + " and " + paths[i]); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                         }
@@ -83,24 +83,30 @@ public final class ClassUtil
                 }
                 catch (IOException ex)
                 {
-                    // If that zip file failed, then ignore it and more on.
+                    // If that zip file failed, then ignore it and move on.
+                    log.warn("Missing zip file for " + classname + " and " + paths[i]); //$NON-NLS-1$ //$NON-NLS-2$
                 }
             }
             else
             {
+                StringBuffer path = new StringBuffer(256);
+
                 // Search for the file
                 String extra = classname.replace('.', File.separatorChar);
 
-                if (!paths[i].endsWith(File.separator))
+                path.append(paths[i]);
+                if (paths[i].charAt(paths[i].length() - 1) != File.separatorChar)
                 {
-                    paths[i] += File.separator;
+                    path.append(File.separatorChar);
                 }
 
-                String file_name = paths[i] + extra + EXTENSION_CLASS;
+                path.append(extra);
+                path.append(EXTENSION_CLASS);
+                String fileName = path.toString();
 
-                if (new File(file_name).isFile())
+                if (new File(fileName).isFile())
                 {
-                    if (full != null && !full.equals(file_name))
+                    if (full != null && !full.equals(fileName))
                     {
                         log.warn("Warning duplicate " + classname + " found: " + full + " and " + paths[i]); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                     }
@@ -155,7 +161,7 @@ public final class ClassUtil
                         log.warn("Class " + impl.getName() + " does not implement " + clazz.getName() + ". Ignoring."); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                     }
                 }
-                catch (Exception ex)
+                catch (ClassNotFoundException ex)
                 {
                     log.warn("Failed to add class to list: " + clazz.getName(), ex); //$NON-NLS-1$
                 }
@@ -164,7 +170,7 @@ public final class ClassUtil
             log.debug("Found " + matches.size() + " implementors of " + clazz.getName()); //$NON-NLS-1$ //$NON-NLS-2$
             return (Class[]) matches.toArray(new Class[matches.size()]);
         }
-        catch (Exception ex)
+        catch (IOException ex)
         {
             log.error("Failed to get any classes.", ex); //$NON-NLS-1$
             return new Class[0];
@@ -206,7 +212,7 @@ public final class ClassUtil
                         log.warn("Class " + impl.getName() + " does not implement " + clazz.getName() + ". Ignoring."); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                     }
                 }
-                catch (Exception ex)
+                catch (ClassNotFoundException ex)
                 {
                     log.warn("Failed to add class to list: " + clazz.getName(), ex); //$NON-NLS-1$
                 }
@@ -214,7 +220,7 @@ public final class ClassUtil
 
             log.debug("Found " + matches.size() + " implementors of " + clazz.getName()); //$NON-NLS-1$ //$NON-NLS-2$
         }
-        catch (Exception ex)
+        catch (IOException ex)
         {
             log.error("Failed to get any classes.", ex); //$NON-NLS-1$
         }
@@ -263,8 +269,7 @@ public final class ClassUtil
      */
     public static Object getImplementation(Class clazz) throws MalformedURLException, ClassCastException, IOException, ClassNotFoundException, InstantiationException, IllegalAccessException
     {
-        Class impl = getImplementor(clazz);
-        return impl.newInstance();
+        return getImplementor(clazz).newInstance();
     }
 
     /**
