@@ -17,11 +17,10 @@
  * Copyright: 2005
  *     The copyright to this program is held by it's authors.
  *
- * ID: $Id: BookType.java 1086 2006-04-30 23:51:48Z dmsmith $
+ * ID: $Id: BookType.java 1169 2006-10-19 21:48:21Z dmsmith $
  */
 package org.crosswire.jsword.book.sword;
 
-import java.io.File;
 import java.io.Serializable;
 
 import org.crosswire.jsword.book.Book;
@@ -48,9 +47,9 @@ public abstract class BookType implements Serializable
             return new SwordBook(sbmd, backend);
         }
 
-        protected AbstractBackend getBackend(SwordBookMetaData sbmd, File rootPath) throws BookException
+        protected AbstractBackend getBackend(SwordBookMetaData sbmd) throws BookException
         {
-            return new RawBackend(sbmd, rootPath);
+            return new RawBackend(sbmd);
         }
 
         /**
@@ -69,9 +68,9 @@ public abstract class BookType implements Serializable
             return new SwordBook(sbmd, backend);
         }
 
-        protected AbstractBackend getBackend(SwordBookMetaData sbmd, File rootPath) throws BookException
+        protected AbstractBackend getBackend(SwordBookMetaData sbmd) throws BookException
         {
-            return getCompressedBackend(sbmd, rootPath);
+            return getCompressedBackend(sbmd);
         }
 
         protected boolean isBackendSupported(SwordBookMetaData sbmd)
@@ -95,9 +94,9 @@ public abstract class BookType implements Serializable
             return new SwordBook(sbmd, backend);
         }
 
-        protected AbstractBackend getBackend(SwordBookMetaData sbmd, File rootPath) throws BookException
+        protected AbstractBackend getBackend(SwordBookMetaData sbmd) throws BookException
         {
-            return new RawBackend(sbmd, rootPath);
+            return new RawBackend(sbmd);
         }
 
         /**
@@ -116,9 +115,9 @@ public abstract class BookType implements Serializable
             return new SwordBook(sbmd, backend);
         }
 
-        protected AbstractBackend getBackend(SwordBookMetaData sbmd, File rootPath) throws BookException
+        protected AbstractBackend getBackend(SwordBookMetaData sbmd) throws BookException
         {
-            return getCompressedBackend(sbmd, rootPath);
+            return getCompressedBackend(sbmd);
         }
 
         protected boolean isBackendSupported(SwordBookMetaData sbmd)
@@ -142,9 +141,9 @@ public abstract class BookType implements Serializable
             return new SwordBook(sbmd, backend);
         }
 
-        protected AbstractBackend getBackend(SwordBookMetaData sbmd, File rootPath) throws BookException
+        protected AbstractBackend getBackend(SwordBookMetaData sbmd) throws BookException
         {
-            return new RawBackend(sbmd, rootPath);
+            return new RawBackend(sbmd);
         }
 
         /**
@@ -163,9 +162,9 @@ public abstract class BookType implements Serializable
             return new SwordBook(sbmd, backend);
         }
 
-        protected AbstractBackend getBackend(SwordBookMetaData sbmd, File rootPath) throws BookException
+        protected AbstractBackend getBackend(SwordBookMetaData sbmd) throws BookException
         {
-            return new RawBackend(sbmd, rootPath);
+            return new RawBackend(sbmd);
         }
 
         /**
@@ -188,9 +187,9 @@ public abstract class BookType implements Serializable
             return new SwordDictionary(sbmd, backend);
         }
 
-        protected AbstractBackend getBackend(SwordBookMetaData sbmd, File rootPath) throws BookException
+        protected AbstractBackend getBackend(SwordBookMetaData sbmd) throws BookException
         {
-            return new RawLDBackend(sbmd, rootPath, 2);
+            return new RawLDBackend(sbmd, 2);
         }
 
         /**
@@ -213,9 +212,9 @@ public abstract class BookType implements Serializable
             return new SwordDictionary(sbmd, backend);
         }
 
-        protected AbstractBackend getBackend(SwordBookMetaData sbmd, File rootPath) throws BookException
+        protected AbstractBackend getBackend(SwordBookMetaData sbmd) throws BookException
         {
-            return new RawLDBackend(sbmd, rootPath, 4);
+            return new RawLDBackend(sbmd, 4);
         }
 
         /**
@@ -238,14 +237,9 @@ public abstract class BookType implements Serializable
             return new SwordDictionary(sbmd, backend);
         }
 
-        protected AbstractBackend getBackend(SwordBookMetaData sbmd, File rootPath) throws BookException
+        protected AbstractBackend getBackend(SwordBookMetaData sbmd) throws BookException
         {
-            return new ZLDBackend(sbmd, rootPath);
-        }
-
-        protected boolean isBackendSupported(SwordBookMetaData sbmd)
-        {
-            return false;
+            return new ZLDBackend(sbmd);
         }
 
         /**
@@ -264,9 +258,9 @@ public abstract class BookType implements Serializable
             return new SwordBook(sbmd, backend);
         }
 
-        protected AbstractBackend getBackend(SwordBookMetaData sbmd, File rootPath) throws BookException
+        protected AbstractBackend getBackend(SwordBookMetaData sbmd) throws BookException
         {
-            return new GenBookBackend(sbmd, rootPath);
+            return new GenBookBackend(sbmd);
         }
 
         protected boolean isBackendSupported(SwordBookMetaData sbmd)
@@ -340,11 +334,9 @@ public abstract class BookType implements Serializable
      * Create a Book appropriate for the BookMetaData
      * @throws BookException
      */
-    public Book createBook(SwordBookMetaData sbmd, File progdir) throws BookException
+    public Book createBook(SwordBookMetaData sbmd) throws BookException
     {
-        AbstractBackend backend = getBackend(sbmd, progdir);
-        Book book = getBook(sbmd, backend);
-        return book;
+        return getBook(sbmd, getBackend(sbmd));
     }
 
     /**
@@ -355,19 +347,20 @@ public abstract class BookType implements Serializable
     /**
      * Create a the appropriate backend for this type of book
      */
-    protected abstract AbstractBackend getBackend(SwordBookMetaData sbmd, File rootPath) throws BookException;
+    protected abstract AbstractBackend getBackend(SwordBookMetaData sbmd) throws BookException;
 
     /**
      * 
      */
-    protected static AbstractBackend getCompressedBackend(SwordBookMetaData sbmd, File rootPath) throws BookException
+    protected static AbstractBackend getCompressedBackend(SwordBookMetaData sbmd) throws BookException
     {
         String cStr = sbmd.getProperty(ConfigEntryType.COMPRESS_TYPE);
         if (cStr != null)
         {
-            return CompressionType.fromString(cStr).getBackend(sbmd, rootPath);
+            return CompressionType.fromString(cStr).getBackend(sbmd);
         }
-        throw new BookException(Msg.COMPRESSION_UNSUPPORTED, new Object[] { cStr });
+        assert false;
+        throw new BookException(Msg.COMPRESSION_UNSUPPORTED, new Object[] { "no compression given" }); //$NON-NLS-1$
     }
 
     /**
