@@ -17,7 +17,7 @@
  * Copyright: 2005
  *     The copyright to this program is held by it's authors.
  *
- * ID: $Id: ConfigEntryTable.java 1109 2006-07-24 23:40:27Z dmsmith $
+ * ID: $Id: ConfigEntryTable.java 1189 2006-11-18 13:22:36Z dmsmith $
  */
 package org.crosswire.jsword.book.sword;
 
@@ -62,73 +62,106 @@ import org.jdom.Element;
  * @author Jacky Cheung
  * @author DM Smith [dmsmith555 at yahoo dot com]
  */
-public class ConfigEntryTable
+public final class ConfigEntryTable
 {
     /**
-     * Loads a sword config from a given file.
-     * @throws IOException
+     * Create an empty Sword config for the named book.
+     * @param bookName the name of the book
      */
-    public ConfigEntryTable(File file, String bookName) throws IOException
+    public ConfigEntryTable(String bookName)
     {
-        configFile = file;
+        table = new HashMap();
         internal = bookName;
         supported = true;
-        table = new HashMap();
-
-        BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(file), ENCODING_LATIN1));
-        loadInitials(in);
-        loadContents(in);
-        in.close();
-        if (getValue(ConfigEntryType.ENCODING).equals(ENCODING_UTF8))
-        {
-            supported = true;
-            bookType = null;
-            questionable = false;
-            readahead = null;
-            table.clear();
-            in = new BufferedReader(new InputStreamReader(new FileInputStream(file), ENCODING_UTF8));
-            loadInitials(in);
-            loadContents(in);
-            in.close();
-        }
-        adjustLanguage();
-        adjustBookType();
-        adjustName();
-        validate();
     }
 
     /**
-     * Loads a sword config from a given buffer.
-     * This is used to load conf entries from the mods.d.tar.gz file.
-     * 
+     * Load the conf from a file.
+     * @param file the file to load
      * @throws IOException
      */
-    public ConfigEntryTable(byte[] buffer, String bookName) throws IOException
+    public void load(File file) throws IOException
     {
-        internal = bookName;
-        supported = true;
-        table = new HashMap();
+        configFile = file;
 
-        BufferedReader in = new BufferedReader(new InputStreamReader(new ByteArrayInputStream(buffer), ENCODING_LATIN1));
-        loadInitials(in);
-        loadContents(in);
-        in.close();
-        if (getValue(ConfigEntryType.ENCODING).equals(ENCODING_UTF8))
+        BufferedReader in = null;
+        try
         {
-            supported = true;
-            bookType = null;
-            questionable = false;
-            readahead = null;
-            table.clear();
-            in = new BufferedReader(new InputStreamReader(new ByteArrayInputStream(buffer), ENCODING_UTF8));
+            in = new BufferedReader(new InputStreamReader(new FileInputStream(file), ENCODING_LATIN1));
             loadInitials(in);
             loadContents(in);
             in.close();
+            in = null;
+            if (getValue(ConfigEntryType.ENCODING).equals(ENCODING_UTF8))
+            {
+                supported = true;
+                bookType = null;
+                questionable = false;
+                readahead = null;
+                table.clear();
+                in = new BufferedReader(new InputStreamReader(new FileInputStream(file), ENCODING_UTF8));
+                loadInitials(in);
+                loadContents(in);
+                in.close();
+                in = null;
+            }
+            adjustDataPath();
+            adjustLanguage();
+            adjustBookType();
+            adjustName();
+            validate();
+        }
+        finally
+        {
+            if (in != null)
+            {
+                in.close();
+            }
+        }
+    }
+
+    /**
+     * Load the conf from a buffer.
+     * This is used to load conf entries from the mods.d.tar.gz file.
+     * @param buffer the buffer to load
+     * @throws IOException
+     */
+    public void load(byte[] buffer) throws IOException
+    {
+        BufferedReader in = null;
+        try
+        {
+            in = new BufferedReader(new InputStreamReader(new ByteArrayInputStream(buffer), ENCODING_LATIN1));
+            loadInitials(in);
+            loadContents(in);
+            in.close();
+            in = null;
+            if (getValue(ConfigEntryType.ENCODING).equals(ENCODING_UTF8))
+            {
+                supported = true;
+                bookType = null;
+                questionable = false;
+                readahead = null;
+                table.clear();
+                in = new BufferedReader(new InputStreamReader(new ByteArrayInputStream(buffer), ENCODING_UTF8));
+                loadInitials(in);
+                loadContents(in);
+                in.close();
+                in = null;
+            }
+            adjustDataPath();
+            adjustLanguage();
+            adjustBookType();
+            adjustName();
+            validate();
+        }
+        finally
+        {
+            if (in != null)
+            {
+                in.close();
+            }
         }
-        adjustLanguage();
-        adjustBookType();
-        adjustName();
-        validate();
     }
 
     /**
@@ -176,11 +209,12 @@ public class ConfigEntryTable
      */
     public boolean unlock(String unlockKey)
     {
-        if (unlockKey != null)
+        String tmpKey = unlockKey;
+        if (tmpKey != null)
         {
-            unlockKey = unlockKey.trim();
+            tmpKey = tmpKey.trim();
         }
-        add(ConfigEntryType.CIPHER_KEY, unlockKey);
+        add(ConfigEntryType.CIPHER_KEY, tmpKey);
         if (configFile != null)
         {
             try
@@ -277,8 +311,7 @@ public class ConfigEntryTable
         StringBuffer buf = new StringBuffer();
         buf.append('[');
         buf.append(getValue(ConfigEntryType.INITIALS));
-        buf.append(']');
-        buf.append('\n');
+        buf.append("]\n"); //$NON-NLS-1$
         toConf(buf, BASIC_INFO);
         toConf(buf, SYSTEM_INFO);
         toConf(buf, HIDDEN);
@@ -298,9 +331,19 @@ public class ConfigEntryTable
             {
                 encoding = ENCODING_UTF8;
             }
-            Writer writer = new OutputStreamWriter(new FileOutputStream(configFile), encoding);
-            writer.write(toConf());
-            writer.close();
+            Writer writer = null;
+            try
+            {
+                writer = new OutputStreamWriter(new FileOutputStream(configFile), encoding);
+                writer.write(toConf());
+            }
+            finally
+            {
+                if (writer != null)
+                {
+                    writer.close();
+                }
+            }
         }
     }
 
@@ -324,7 +367,7 @@ public class ConfigEntryTable
                 continue;
             }
 
-            int eqpos = line.indexOf("="); //$NON-NLS-1$
+            int eqpos = line.indexOf('=');
             if (eqpos == -1)
             {
                 log.warn("Expected to see '=' in " + internal + ": " + line); //$NON-NLS-1$ //$NON-NLS-2$
@@ -432,7 +475,7 @@ public class ConfigEntryTable
             {
                 if (continuation_expected)
                 {
-                    log.warn("Continuation followed by key for " + configEntry.getName() + " in " + internal + ": " + line); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
+                    log.warn(report("Continuation followed by key for", configEntry.getName(), line)); //$NON-NLS-1$
                 }
 
                 backup(line);
@@ -440,12 +483,12 @@ public class ConfigEntryTable
             }
             else if (!continuation_expected)
             {
-                log.warn("Line without previous continuation for " + configEntry.getName()  + " in " + internal + ": " + line); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
+                log.warn(report("Line without previous continuation for", configEntry.getName(), line)); //$NON-NLS-1$
             }
 
             if (!configEntry.allowsContinuation())
             {
-                log.warn("Ignoring unexpected additional line for " + configEntry.getName()  + " in " + internal + ": " + line); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
+                log.warn(report("Ignoring unexpected additional line for", configEntry.getName(), line)); //$NON-NLS-1$
             }
             else
             {
@@ -521,11 +564,31 @@ public class ConfigEntryTable
      * @param type
      * @param aValue
      */
-    private void add(ConfigEntryType type, String aValue)
+    public void add(ConfigEntryType type, String aValue)
     {
         table.put(type, new ConfigEntry(internal, type, aValue));
     }
 
+    private void adjustDataPath()
+    {
+        String datapath = (String) getValue(ConfigEntryType.DATA_PATH);
+        if (datapath.startsWith("./")) //$NON-NLS-1$
+        {
+            add(ConfigEntryType.DATA_PATH, datapath.substring(2));
+        }
+    }
+
+    private boolean isLeftToRight(String lang)
+    {
+        // Java does not know that the following languages are right to left
+        if ("fa".equals(lang) || "syr".equals(lang))  //$NON-NLS-1$ //$NON-NLS-2$
+        {
+            return false;
+        }
+
+        return ComponentOrientation.getOrientation(new Locale(lang)).isLeftToRight();
+    }
+
     private void adjustLanguage()
     {
         // Java thinks it is LtoR but it is stated to be something else
@@ -533,19 +596,21 @@ public class ConfigEntryTable
         String newDir = dir == null ? (String) ConfigEntryType.DIRECTION.getDefault() : dir;
 
         String langEntry = (String) getValue(ConfigEntryType.LANG);
+        String langFromEntry = (String) getValue(ConfigEntryType.GLOSSARY_FROM);
+        String langToEntry = (String) getValue(ConfigEntryType.GLOSSARY_TO);
+
+        // The LANG field should match the GLOSSARY_FROM field
+        if (langFromEntry != null && !langFromEntry.equals(langEntry))
+        {
+            langEntry = langFromEntry;
+        }
+
         String lang = AbstractBookMetaData.getLanguage(internal, langEntry);
         add(ConfigEntryType.LANGUAGE, lang);
 
-        // This returns ComponentOrientation.LEFT_TO_RIGHT if
+        // This returns Left to Right if
         // it does not know what it is.
-        boolean leftToRight = true;
-        if (langEntry != null)
-        {
-            leftToRight = ComponentOrientation.getOrientation(new Locale(langEntry)).isLeftToRight();
-        }
-
-        String langFromEntry = (String) getValue(ConfigEntryType.GLOSSARY_FROM);
-        String langToEntry = (String) getValue(ConfigEntryType.GLOSSARY_TO);
+        boolean leftToRight = isLeftToRight(langEntry);
 
         if (langFromEntry != null || langToEntry != null)
         {
@@ -558,20 +623,20 @@ public class ConfigEntryTable
 
             if (langFromEntry == null)
             {
-                log.warn("Missing data for " + internal + ". Assuming " + ConfigEntryType.GLOSSARY_FROM.getName() + "=" + AbstractBookMetaData.DEFAULT_LANG_CODE);  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
+                log.warn("Missing data for " + internal + ". Assuming " + ConfigEntryType.GLOSSARY_FROM.getName() + '=' + AbstractBookMetaData.DEFAULT_LANG_CODE);  //$NON-NLS-1$ //$NON-NLS-2$
             }
             else
             {
-                fromLeftToRight = ComponentOrientation.getOrientation(new Locale(langFromEntry)).isLeftToRight();
+                fromLeftToRight = isLeftToRight(langFromEntry);
             }
 
             if (langToEntry == null)
             {
-                log.warn("Missing data for " + internal + ". Assuming " + ConfigEntryType.GLOSSARY_TO.getName() + "=" + AbstractBookMetaData.DEFAULT_LANG_CODE);  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
+                log.warn("Missing data for " + internal + ". Assuming " + ConfigEntryType.GLOSSARY_TO.getName() + '=' + AbstractBookMetaData.DEFAULT_LANG_CODE);  //$NON-NLS-1$ //$NON-NLS-2$
             }
             else
             {
-                toLeftToRight = ComponentOrientation.getOrientation(new Locale(langToEntry)).isLeftToRight();
+                toLeftToRight = isLeftToRight(langToEntry);
             }
 
             // At least one of the two languages should match the lang entry
@@ -746,6 +811,20 @@ public class ConfigEntryTable
             }
         }
     }
+    private String report(String issue, String confEntryName, String line)
+    {
+        StringBuffer buf = new StringBuffer(100);
+        buf.append(issue);
+        buf.append(' ');
+        buf.append(confEntryName);
+        buf.append(" in "); //$NON-NLS-1$
+        buf.append(internal);
+        buf.append(": "); //$NON-NLS-1$
+        buf.append(line);
+
+        return buf.toString();
+    }
+
 
    /**
      * Sword only recognizes two encodings for its modules: UTF-8 and LATIN1
@@ -758,7 +837,8 @@ public class ConfigEntryTable
      * These are the elements that JSword requires.
      * They are a superset of those that Sword requires.
      */
-    public static final ConfigEntryType[] REQUIRED =
+    /* For documentation purposes at this time.
+    private static final ConfigEntryType[] REQUIRED =
     {
         ConfigEntryType.INITIALS,
         ConfigEntryType.DESCRIPTION,
@@ -766,19 +846,21 @@ public class ConfigEntryTable
         ConfigEntryType.DATA_PATH,
         ConfigEntryType.MOD_DRV,
     };
+    */
 
-    public static final ConfigEntryType[] BASIC_INFO =
+    private static final ConfigEntryType[] BASIC_INFO =
     {
         ConfigEntryType.INITIALS,
         ConfigEntryType.DESCRIPTION,
         ConfigEntryType.CATEGORY,
         ConfigEntryType.LCSH,
-        ConfigEntryType.VERSION,
         ConfigEntryType.SWORD_VERSION_DATE,
+        ConfigEntryType.VERSION,
         ConfigEntryType.HISTORY,
+        ConfigEntryType.OBSOLETES,
     };
 
-    public static final ConfigEntryType[] LANG_INFO =
+    private static final ConfigEntryType[] LANG_INFO =
     {
         ConfigEntryType.LANGUAGE,
         ConfigEntryType.LANG,
@@ -788,7 +870,7 @@ public class ConfigEntryTable
         ConfigEntryType.GLOSSARY_TO,
     };
 
-    public static final ConfigEntryType[] COPYRIGHT_INFO =
+    private static final ConfigEntryType[] COPYRIGHT_INFO =
     {
         ConfigEntryType.ABOUT,
         ConfigEntryType.SHORT_PROMO,
@@ -802,18 +884,19 @@ public class ConfigEntryTable
         ConfigEntryType.COPYRIGHT_CONTACT_NAME,
         ConfigEntryType.COPYRIGHT_CONTACT_ADDRESS,
         ConfigEntryType.COPYRIGHT_CONTACT_EMAIL,
+        ConfigEntryType.COPYRIGHT_CONTACT_NOTES,
         ConfigEntryType.COPYRIGHT_NOTES,
         ConfigEntryType.TEXT_SOURCE,
     };
 
-    public static final ConfigEntryType[] FEATURE_INFO =
+    private static final ConfigEntryType[] FEATURE_INFO =
     {
         ConfigEntryType.FEATURE,
         ConfigEntryType.GLOBAL_OPTION_FILTER,
         ConfigEntryType.FONT,
     };
 
-    public static final ConfigEntryType[] SYSTEM_INFO =
+    private static final ConfigEntryType[] SYSTEM_INFO =
     {
         ConfigEntryType.DATA_PATH,
         ConfigEntryType.MOD_DRV,
@@ -822,10 +905,11 @@ public class ConfigEntryTable
         ConfigEntryType.COMPRESS_TYPE,
         ConfigEntryType.ENCODING,
         ConfigEntryType.MINIMUM_VERSION,
+        ConfigEntryType.OSIS_VERSION,
         ConfigEntryType.DIRECTION,
     };
 
-    public static final ConfigEntryType[] HIDDEN =
+    private static final ConfigEntryType[] HIDDEN =
     {
         ConfigEntryType.CIPHER_KEY,
     };
