@@ -17,17 +17,19 @@
  * Copyright: 2005
  *     The copyright to this program is held by it's authors.
  *
- * ID: $Id: VerseRange.java 1068 2006-04-08 02:20:41Z dmsmith $
+ * ID: $Id: VerseRange.java 1178 2006-11-06 12:48:02Z dmsmith $
  */
 package org.crosswire.jsword.passage;
 
 import java.io.IOException;
 import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
+import java.io.Serializable;
 import java.util.Iterator;
 import java.util.NoSuchElementException;
 
 import org.crosswire.common.util.Logger;
+import org.crosswire.jsword.versification.BibleInfo;
 
 /**
  * A VerseRange is one step between a Verse and a Passage - it is a
@@ -42,7 +44,7 @@ import org.crosswire.common.util.Logger;
  * @author Joe Walker [joe at eireneh dot com]
  * @author DM Smith [dmsmith555 at yahoo dot com]
  */
-public final class VerseRange implements Key
+public final class VerseRange implements Key, Serializable
 {
     /**
      * The default VerseRange is a single verse - Genesis 1:1. I didn't
@@ -147,17 +149,17 @@ public final class VerseRange implements Key
         }
 
         // Cache these we're going to be using them a lot.
-        int start_book = start.getBook();
-        int start_chapter = start.getChapter();
-        int start_verse = start.getVerse();
-        int end_book = end.getBook();
-        int end_chapter = end.getChapter();
-        int end_verse = end.getVerse();
+        int startBook = start.getBook();
+        int startChapter = start.getChapter();
+        int startVerse = start.getVerse();
+        int endBook = end.getBook();
+        int endChapter = end.getChapter();
+        int endVerse = end.getVerse();
 
         try
         {
             // If this is in 2 separate books
-            if (start_book != end_book)
+            if (startBook != endBook)
             {
                 // This range is exactly a whole book
                 if (isWholeBooks())
@@ -165,19 +167,19 @@ public final class VerseRange implements Key
                     // Just report the name of the book, we don't need to worry about the
                     // base since we start at the start of a book, and should have been
                     // recently normalized()
-                    return BibleInfo.getBookName(start_book)
+                    return BibleInfo.getBookName(startBook)
                          + VerseRange.RANGE_PREF_DELIM
-                         + BibleInfo.getBookName(end_book);
+                         + BibleInfo.getBookName(endBook);
                 }
 
                 // If this range is exactly a whole chapter
                 if (isWholeChapters())
                 {
                     // Just report book and chapter names
-                    return BibleInfo.getBookName(start_book)
-                         + Verse.VERSE_PREF_DELIM1 + start_chapter
-                         + VerseRange.RANGE_PREF_DELIM + BibleInfo.getBookName(end_book)
-                         + Verse.VERSE_PREF_DELIM1 + end_chapter;
+                    return BibleInfo.getBookName(startBook)
+                         + Verse.VERSE_PREF_DELIM1 + startChapter
+                         + VerseRange.RANGE_PREF_DELIM + BibleInfo.getBookName(endBook)
+                         + Verse.VERSE_PREF_DELIM1 + endChapter;
                 }
 
                 return start.getName(base) + VerseRange.RANGE_PREF_DELIM + end.getName(base);
@@ -189,39 +191,39 @@ public final class VerseRange implements Key
                 // Just report the name of the book, we don't need to worry about the
                 // base since we start at the start of a book, and should have been
                 // recently normalized()
-                return BibleInfo.getBookName(start_book);
+                return BibleInfo.getBookName(startBook);
             }
 
             // If this is 2 separate chapters
-            if (start_chapter != end_chapter)
+            if (startChapter != endChapter)
             {
                 // If this range is a whole number of chapters
                 if (isWholeChapters())
                 {
                     // Just report the name of the book and the chapters
-                    return BibleInfo.getBookName(start_book)
-                         + Verse.VERSE_PREF_DELIM1 + start_chapter
-                         + VerseRange.RANGE_PREF_DELIM + end_chapter;
+                    return BibleInfo.getBookName(startBook)
+                         + Verse.VERSE_PREF_DELIM1 + startChapter
+                         + VerseRange.RANGE_PREF_DELIM + endChapter;
                 }
 
                 return start.getName(base)
-                     + VerseRange.RANGE_PREF_DELIM + end_chapter
-                     + Verse.VERSE_PREF_DELIM2 + end_verse;
+                     + VerseRange.RANGE_PREF_DELIM + endChapter
+                     + Verse.VERSE_PREF_DELIM2 + endVerse;
             }
 
             // If this range is exactly a whole chapter
             if (isWholeChapter())
             {
                 // Just report the name of the book and the chapter
-                return BibleInfo.getBookName(start_book)
-                     + Verse.VERSE_PREF_DELIM1 + start_chapter;
+                return BibleInfo.getBookName(startBook)
+                     + Verse.VERSE_PREF_DELIM1 + startChapter;
             }
 
             // If this is 2 separate verses
-            if (start_verse != end_verse)
+            if (startVerse != endVerse)
             {
                 return start.getName(base)
-                     + VerseRange.RANGE_PREF_DELIM + end_verse;
+                     + VerseRange.RANGE_PREF_DELIM + endVerse;
             }
 
             // The range is a single verse
@@ -449,17 +451,16 @@ public final class VerseRange implements Key
         try
         {
             copy = (VerseRange) super.clone();
+            copy.start = (Verse) start.clone();
+            copy.end = (Verse) end.clone();
+            copy.verseCount = verseCount;
+            copy.originalName = originalName;
         }
         catch (CloneNotSupportedException e)
         {
             assert false : e;
         }
 
-        copy.start = (Verse) start.clone();
-        copy.end = (Verse) end.clone();
-        copy.verseCount = verseCount;
-        copy.originalName = originalName;
-
         return copy;
     }
 
@@ -1133,9 +1134,9 @@ public final class VerseRange implements Key
     public static final String RANGE_ALLOWED_DELIMS = "-"; //$NON-NLS-1$
 
     /**
-     * What characters should we use to separate VerseRange parts
+     * What characters should we use to separate VerseRange parts on output
      */
-    public static final String RANGE_PREF_DELIM = "-"; //$NON-NLS-1$
+    public static final String RANGE_PREF_DELIM = RANGE_ALLOWED_DELIMS;
 
     /**
      * To make serialization work across new versions
